/*****************************************************************************
** igluErrorExit.cpp                                                        **
** ------------                                                             **
**                                                                          **
** A file that defines common functions for crashing or printing fatal      **
**    error messages inside of IGLU.                                        **
**                                                                          **
** Chris Wyman (2/23/2012)                                                  **
*****************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cctype>
#include "iglu.h"

using namespace iglu;

void iglu::ErrorExit( const char *error, 
			          const char *fileName, const char *funcName, int lineNum )
{
	printf("*** Encountered Fatal IGLU Error!!\n");
	printf("    -> Line: %4d,  File: [%s]\n", lineNum, fileName );
	printf("    ->          Function: [%s]\n", funcName); 
	printf("\n*** Detailed Message Follows:\n\n%s\n", error);
	exit(-1);
}

void iglu::Warning( const char *error, 
			          const char *fileName, const char *funcName, int lineNum )
{
	printf("*** WARNING.  Undesirable IGLU behavior!  Consider code modifications!\n");
	printf("    -> Line: %4d,  File: [%s]\n", lineNum, fileName );
	printf("    ->          Function: [%s]\n", funcName); 
	printf("*** Detailed Message Follows:\n%s\n", error);
}

bool iglu::CheckValidOpenGL( const char *fileName, const char *funcName, int lineNum )
{
	// We know we have a valid context after glewInit() executes successfully, and
	//     we have access to functions that are loaded from vendor-specific GL DLLs
	//if( !glGetString( GL_VERSION ) )     // non-NULL only after glewInit()
	if( !glGenerateMipmap )            // non-NULL only after glewInit()
	{
		printf("*** ERROR!  %s called at line %d of %s\n", funcName, lineNum, fileName);
		printf("            This function requires a valid OpenGL context, but no context is\n");
		printf("            currently available!  Typically, no OpenGL context is available\n");
		printf("            until a GL window is created.  (Some IGLU objects, like textures\n");
		printf("            can be created without a context using delayed initialization;\n");
		printf("            see the appropriate class header and constructor.)\n");
		exit(-1);  
		return false;
	}

	return true;
}


void iglu::StatusMessage( const char *error )
{
	printf("IGLU Status: %s\n", error);
}

void iglu::Warn( const char *error )
{
	printf("Warning! %s\n", error);
}

void iglu::LoadError ( const char *fileName, const char *inFunction )
{
	printf("*** Error: %s unable to load file: '%s'\n", inFunction, fileName);
	exit(-1);
}

