/******************************************************************/
/* igluFilePath.cpp                                               */
/* -----------------------                                        */
/*                                                                */
/* A header for interfacing with a simplistic filepath manager    */
/*     and an IGLU version of fopen() for loading files from      */
/*     specified path directories.                                */
/*                                                                */
/* Chris Wyman (10/20/2011)                                       */
/******************************************************************/

#include "iglu.h"
#include <sys/types.h>
#include <sys/stat.h>

using namespace iglu;

IGLUFileIO::FileType IGLUFileIO::GetFileType( char *fName )
{
	// Identify the type of file
	char *ptr = strrchr( fName, '.' );
	char buf[16];
	strncpy( buf, ptr, 16 );
	for (int i=0;i<16;i++) buf[i] = tolower( buf[i] );

	if (!strcmp(buf, ".ppm") || !strcmp(buf, ".pgm"))	
		return IGLUFileIO::TYPE_PPM;
	else if (!strcmp(buf, ".pfm") || !strcmp(buf, ".pf4"))	
		return IGLUFileIO::TYPE_PFM;
	else if (!strcmp(buf, ".rgb") || !strcmp(buf, ".rgba"))
		return IGLUFileIO::TYPE_RGB;
	else if (!strcmp(buf, ".bmp"))
		return IGLUFileIO::TYPE_BMP;
	else if (!strcmp(buf, ".jpg") || !strcmp(buf, ".jpeg"))
		return IGLUFileIO::TYPE_JPG;
	else if (!strcmp(buf, ".png"))
		return IGLUFileIO::TYPE_PNG;
	else if (!strcmp(buf, ".tga"))
		return IGLUFileIO::TYPE_TGA;
	else if (!strcmp(buf, ".obj"))
		return IGLUFileIO::TYPE_OBJ;
	else if (!strcmp(buf, ".objar"))
		return IGLUFileIO::TYPE_OBJAR;

	return IGLUFileIO::TYPE_UNKNOWN;
}
