/******************************************************************/
/* igluCallbackFileParser.cpp                                     */
/* -----------------------                                        */
/*                                                                */
/* This class provides a callback-based approach for parsing      */
/*    simple text files.  You pass in a filename, setup a list of */
/*    callback functions to execute when encountering lines that  */
/*    with certain tokens, then call Parse().                     */
/*                                                                */
/* Chris Wyman (06/09/2010)                                       */
/******************************************************************/

#include "iglu.h"

using namespace iglu;

#pragma warning( disable : 4996 )

IGLUCallbackFileParser::IGLUCallbackFileParser( const char *filename, int searchPathType ) : 
	IGLUFileParser( filename, searchPathType )
{
}

IGLUCallbackFileParser::~IGLUCallbackFileParser()
{
	for (uint i=0; i<callbackTokens.Size(); i++)
		free( callbackTokens[i] );
}

void IGLUCallbackFileParser::SetCallback( char *token, Callback func )
{
	uint i = callbackTokens.Add( strdup( token ) );
	MakeLower( callbackTokens[i] );
	callbacks.Add( func );
}

void IGLUCallbackFileParser::Parse( void )
{
	char *linePtr = 0;
	char keyword[512];

	// For each line in the file... (Except those processed by callbacks inside the loop)
	while ( (linePtr = this->ReadNextLine()) != NULL )
	{
		// Each line starts with a keyword.  Get this token
		this->GetLowerCaseToken( keyword );

		// Check all of our callbacks to see if this line matches
		for (uint i=0; i<callbackTokens.Size(); i++)
			if ( !strcmp(keyword, callbackTokens[i]) )  // We found a matching token!
				(*callbacks[i])( this );                //    Execute the callback.

		// If no matching token/callback pairs were found, ignore the line.
	}

	// Done parsing the file.  Close it.
	CloseFile();
}