/************************************************************************/
/* igluCameraRotator.cpp                                                */
/* ------------------                                                   */
/*                                                                      */
/* This class implements a simple rotator around the eye position, used */
/*    for mouse rotations of the camera view point.                     */
/*                                                                      */
/* Inputs are:                                                          */
/*    * Integers width & height.  To rotate in a plausible manner, the  */
/*      rotator needs the correct window (or subwindow) width & height. */
/*                                                                      */
/* Chris Wyman (01/14/2013)                                             */
/************************************************************************/


#include "iglu.h"

using namespace iglu;


IGLUCameraRotator::IGLUCameraRotator( int width, int height ) : 
	IGLUMouseInteractor(width,height)
{
	Reset();
}

void IGLUCameraRotator::SetOnClick( int x, int y )
{
	m_lastPos[0] = float(x);
	m_lastPos[1] = float(y);
	m_lastPos[2] = 0.0f;
	m_currentlyTracking = 1;
}

float IGLUCameraRotator::UpdateOnMotion( int x, int y )
{
	float dx, dy, angleX=0, angleY=0;
	if (!m_currentlyTracking) return 0;

    dx = x - m_lastPos[0];
	dy = y - m_lastPos[1];

	if ( fabs(dx) > 0 || fabs(dy) > 0 )
	{
		angleY          = 45.0f * (dx / m_width);   // Rotation around Y-axis
		angleX          = 45.0f * (dy / m_height);  // Rotation around X-axis
		m_lastPos[0]    = float(x);
		m_lastPos[1]    = float(y);
		m_matrix        = IGLUMatrix4x4::Rotate( angleX, vec3::XAxis() ) * IGLUMatrix4x4::Rotate( angleY, vec3::YAxis() );
		m_inverseMatrix = m_matrix.Invert();
		m_hasChanged    = 1;
	}

	return sqrtf( angleX*angleX + angleY*angleY );
}

void IGLUCameraRotator::Release( void )                    
{
	m_currentlyTracking = 0; 
}

void IGLUCameraRotator::Reset( void )
{
	m_currentlyTracking = 0;
	m_lastPos[0] = m_lastPos[1] = m_lastPos[2] = 0; 
	m_matrix = IGLUMatrix4x4::Identity();
	m_inverseMatrix = IGLUMatrix4x4::Identity();
	m_hasChanged = 1;
}


